# -*- coding: utf-8 -*-
import os

from django.conf import settings
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.translation import ugettext_lazy as _

from model_utils.models import TimeStampedModel

from . import choices, managers, utils


def file_path(instance, filename):
    return 'article-pdfs/{0}'.format(filename)


class Article(TimeStampedModel):
    """Article from NCBI. PubMed or PMC."""

    source = models.SmallIntegerField(
        choices=choices.SOURCES,
        default=choices.SOURCES.pubmed
    )
    file = models.FileField(upload_to=file_path, blank=True, null=True)
    identifier = models.IntegerField(db_index=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        blank=True,
        null=True
    )
    active = models.BooleanField(default=True)
    description = models.CharField(max_length=255, blank=True)
    comments = GenericRelation('user_activities.Comment')

    objects = managers.ArticleQuerySet.as_manager()

    class Meta:
        verbose_name = _('Article')
        verbose_name_plural = _('Articles')
        unique_together = ('source', 'identifier')

    def __str__(self):
        return '{0}: {1}'.format(self.get_source_display(), self.identifier)

    @property
    def article_url(self):
        return utils.build_article_url(self.identifier, self.get_source_display())

    @property
    def fetch_url(self):
        return utils.build_efetch_url(self.identifier, self.get_source_display())

    @property
    def summary_url(self):
        return utils.build_esummary_url(self.identifier, self.get_source_display())

    @property
    def file_url(self):
        return os.path.join(settings.MEDIA_URL, str(self.file))



import os


NCBI_URL = 'http://www.ncbi.nlm.nih.gov'

EUTILS_URL = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils'


def build_article_url(id, source='pubmed'):
    if source.lower() == 'pmc':
        return os.path.join(NCBI_URL, source, 'articles', 'PMC{0}'.format(id))
    else:
        return os.path.join(NCBI_URL, source, str(id))


def build_efetch_url(id, source='pubmed', mode='xml'):
    params = '?db={0}&id={1}&retmode={2}'.format(source, str(id), mode)
    return os.path.join(EUTILS_URL, 'efetch.fcgi', params)


def build_esummary_url(id, source='pubmed', mode='json'):
    params = '?db={0}&id={1}&retmode={2}'.format(source, str(id), mode)
    return os.path.join(EUTILS_URL, 'esummary.fcgi', params)



import xmltodict


def parse_efetch(efetch, esummary, identifier):
    """Parses XML response from eutils efetch API

    Arguments:
        efetch (str): XML from efetch
        esummary (dict): JSON from esummary
        identifier (int): eutils identifier

    Returns:
        dict: Python dict with details of article
    """

    efetch = xmltodict.parse(efetch)
    esummary = esummary.get('result', {}).get(str(identifier), {})

    title = esummary.get('title')
    journal = esummary.get('source')
    date = esummary.get('pubdate')
    first_author = esummary.get('sortfirstauthor')

    authors = []
    for author in esummary.get('authors', []):
        authors.append(author.get('name'))

    abstract_obj = efetch \
        .get('PubmedArticleSet', {}) \
        .get('PubmedArticle', {}) \
        .get('MedlineCitation', {}) \
        .get('Article', {}) \
        .get('Abstract', {})

    abstract = ''
    for key, value in abstract_obj.items():
        if key == 'AbstractText':
            if type(value) == str:
                abstract += '{0} '.format(value)

            if type(value) == list:
                for item in value:
                    for text_key, text in item.items():
                        if text_key in ['@Label', '#text']:
                            abstract += '{0} '.format(text)

    return {
        'title': title,
        'journal': journal,
        'date': date,
        'first_author': first_author,
        'authors': authors,
        'abstract': abstract,
    }



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model

from genomix.fields import DisplayChoiceField, UserRelatedField
from rest_framework import serializers

from . import choices, models


class ArticleSerializer(serializers.ModelSerializer):
    """Serializer for Articles."""

    source = DisplayChoiceField(choices=choices.SOURCES)
    user = UserRelatedField(queryset=get_user_model().objects.all())

    class Meta:
        model = models.Article
        fields = (
            'id', 'source', 'file', 'identifier', 'active', 'description',
            'article_url', 'fetch_url', 'summary_url',
            'user', 'created', 'modified',
        )



# -*- coding: utf-8 -*-
from django.conf.urls import url

from rest_framework import routers

from . import views, viewsets


app_name = 'literature_knowledgebase'
router = routers.SimpleRouter()
router.register(r'', viewsets.ArticleViewSet)

default_router = routers.DefaultRouter()
default_router.register(r'articles', viewsets.ArticleViewSet)


urlpatterns = default_router.urls

urlpatterns += [
    url(
        r'^pubmed-articles/(?P<pubmed_id>[0-9]+)/$',
        views.PubmedArticleView.as_view(),
        name='pubmed-articles-detail',
    ),
]



# -*- coding: utf-8 -*-
from collections import OrderedDict
import requests

from rest_framework.exceptions import APIException
from rest_framework.parsers import JSONParser
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK
from rest_framework.views import APIView

from . import services, utils


class PubmedArticleView(APIView):

    parser_classes = (JSONParser, )

    def get(self, request, pubmed_id, **kwargs):
        efetch = requests.get(utils.build_efetch_url(pubmed_id))
        esummary = requests.get(utils.build_esummary_url(pubmed_id))

        if efetch.status_code != 200:
            raise APIException(
                detail='Could not fetch from eutils',
                code=efetch.status_code,
            )

        if esummary.status_code != 200:
            raise APIException(
                detail='Could not fetch from eutils',
                code=esummary.status_code,
            )

        data = services.parse_efetch(
            efetch.text,
            esummary.json(),
            pubmed_id,
        )

        return Response(OrderedDict(data), HTTP_200_OK)



# Generated by Django 2.1.3 on 2018-12-04 21:47

from django.db import migrations, models
import literature_knowledgebase.models


class Migration(migrations.Migration):

    dependencies = [
        ('literature_knowledgebase', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='file',
            field=models.FileField(blank=True, null=True, upload_to=literature_knowledgebase.models.file_path),
        ),
    ]



# -*- coding: utf-8 -*-
from collections import OrderedDict
import requests

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.exceptions import APIException
from rest_framework.filters import SearchFilter
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK

from . import filters, models, serializers, services


class ArticleViewSet(viewsets.ModelViewSet):
    """ViewSet for viewing and editing Articles."""

    queryset = models.Article.objects.fast()
    serializer_class = serializers.ArticleSerializer
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filterset_class = filters.ArticleFilter
    search_fields = ('description', )

    @action(detail=True, methods=['get'], url_path='eutils')
    def eutils(self, request, pk=None):
        instance = self.get_object()
        efetch = requests.get(instance.fetch_url)
        esummary = requests.get(instance.summary_url)

        if efetch.status_code != 200:
            raise APIException(
                detail='Could not fetch from eutils',
                code=efetch.status_code,
            )

        if esummary.status_code != 200:
            raise APIException(
                detail='Could not fetch from eutils',
                code=esummary.status_code,
            )

        data = services.parse_efetch(
            efetch.text,
            esummary.json(),
            instance.identifier
        )

        return Response(OrderedDict(data), HTTP_200_OK)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-09 13:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('source', models.SmallIntegerField(choices=[(0, 'pubmed'), (1, 'pmc')], default=0)),
                ('identifier', models.IntegerField(db_index=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, max_length=255)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Articles',
                'verbose_name': 'Article',
            },
        ),
        migrations.AlterUniqueTogether(
            name='article',
            unique_together=set([('source', 'identifier')]),
        ),
    ]
